@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: if_athvar.h 4099 2009-09-28 23:06:53Z proski $
+ * $Id: if_athvar.h 4118 2010-01-31 07:01:47Z proski $
  */
 
 /*
@@ -984,8 +984,13 @@ typedef void (*ath_callback) (struct ath_softc *);
 #define ATH_GBUF_LOCK_CHECK(_sc)
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
+#include <linux/semaphore.h>
+#else
+#include <asm/semaphore.h>
+#endif
 /* Protects the device from concurrent accesses */
-#define	ATH_LOCK_INIT(_sc)		init_MUTEX(&(_sc)->sc_lock)
+#define	ATH_LOCK_INIT(_sc)		sema_init(&(_sc)->sc_lock, 1)
 #define	ATH_LOCK_DESTROY(_sc)
 #define	ATH_LOCK(_sc)			down(&(_sc)->sc_lock)
 #define	ATH_UNLOCK(_sc)			up(&(_sc)->sc_lock)
