@@ -4,13 +4,18 @@
  * file.
  * Taken from the lm_sensors project.
  *
- * $Id: kernelversion.c 1669 2006-07-05 02:21:30Z proski $
+ * $Id: kernelversion.c 4101 2010-01-09 08:27:17Z proski $
  */
 #include <linux/version.h>
 
-/* Linux 2.6.18+ uses <linux/utsrelease.h> */
 #ifndef UTS_RELEASE
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+/* Linux 2.6.33+ uses <generated/utsrelease.h> */
+#include <generated/utsrelease.h>
+#else
+/* Linux 2.6.18 - 2.6.32 uses <linux/utsrelease.h> */
 #include <linux/utsrelease.h>
 #endif
+#endif
 
 char *uts_release = UTS_RELEASE;
