@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211.c 3902 2009-01-14 02:36:53Z proski $
+ * $Id: ieee80211.c 4078 2009-07-11 17:33:54Z benoit $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -361,6 +361,11 @@ ieee80211_ifattach(struct ieee80211com *ic)
 		ieee80211_expire_dfs_excl_timer;
 	ic->ic_dfs_excl_timer.data = (unsigned long) ic;
 
+	/* Initialize CSA related variables. */
+	init_timer(&ic->ic_csa_timer);
+	ic->ic_csa_timer.data = (unsigned long)ic;
+	ic->ic_csa_timer.function = ieee80211_doth_switch_channel_tmr;
+
 	ieee80211_crypto_attach(ic);
 	ieee80211_node_attach(ic);
 	ieee80211_power_attach(ic);
@@ -400,6 +405,23 @@ ieee80211_ifdetach(struct ieee80211com *ic)
 }
 EXPORT_SYMBOL(ieee80211_ifdetach);
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
+static const struct net_device_ops ieee80211_netdev_ops = {
+	.ndo_get_stats		= ieee80211_getstats,
+	.ndo_open		= ieee80211_open,
+	.ndo_stop		= ieee80211_stop,
+	.ndo_start_xmit		= ieee80211_hardstart,
+	.ndo_set_multicast_list = ieee80211_set_multicast_list,
+	.ndo_change_mtu 	= ieee80211_change_mtu,
+	.ndo_do_ioctl		= ieee80211_ioctl,
+#if IEEE80211_VLAN_TAG_USED
+	.ndo_vlan_rx_register	= ieee80211_vlan_register,
+	.ndo_vlan_rx_add_vid	= ieee80211_vlan_add_vid,
+	.ndo_vlan_rx_kill_vid	= ieee80211_vlan_kill_vid,
+#endif
+};
+#endif
+
 int
 ieee80211_vap_setup(struct ieee80211com *ic, struct net_device *dev,
 	const char *name, int opmode, int flags)
@@ -421,15 +443,22 @@ ieee80211_vap_setup(struct ieee80211com *ic, struct net_device *dev,
 			strncpy(dev->name, name, sizeof(dev->name));
 	}
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
 	dev->get_stats = ieee80211_getstats;
 	dev->open = ieee80211_open;
 	dev->stop = ieee80211_stop;
 	dev->hard_start_xmit = ieee80211_hardstart;
 	dev->set_multicast_list = ieee80211_set_multicast_list;
-#if 0
-	dev->set_mac_address = ieee80211_set_mac_address;
-#endif
  	dev->change_mtu = ieee80211_change_mtu;
+	dev->do_ioctl = ieee80211_ioctl;
+#if IEEE80211_VLAN_TAG_USED
+	dev->vlan_rx_register = ieee80211_vlan_register;
+	dev->vlan_rx_add_vid = ieee80211_vlan_add_vid;
+	dev->vlan_rx_kill_vid = ieee80211_vlan_kill_vid;
+#endif
+#else
+	dev->netdev_ops = &ieee80211_netdev_ops;
+#endif
 	dev->tx_queue_len = 0;			/* NB: bypass queuing */
 	dev->hard_header_len = parent->hard_header_len;
 	/*
@@ -497,8 +526,6 @@ ieee80211_vap_setup(struct ieee80211com *ic, struct net_device *dev,
 	vap->iv_opmode = opmode;
 	IEEE80211_INIT_TQUEUE(&vap->iv_stajoin1tq, ieee80211_sta_join1_tasklet, vap);
 
-	vap->iv_chanchange_count = 0;
-
 	/* Enable various functionality by default, if we're capable. */
 #ifdef ATH_WME	/* Not yet the default */
 	if (vap->iv_caps & IEEE80211_C_WME)
@@ -692,7 +719,8 @@ ieee80211_find_channel(struct ieee80211com *ic, int freq, int flags)
 		c = &ic->ic_channels[i];
 		if (c->ic_freq == freq &&
 		    (flags == 0 || 
-		     (c->ic_flags & IEEE80211_CHAN_ALLTURBO) == flags))
+		     (c->ic_flags & IEEE80211_CHAN_ALLTURBO) ==
+		     (flags & IEEE80211_CHAN_ALLTURBO)))
 			return c;
 	}
 	return NULL;
@@ -845,19 +873,14 @@ ieee80211_dfs_action(struct ieee80211com *ic) {
 		/* It is not scanning, but waiting for ath driver to move the 
 		 * vap to RUN. */
 	}
-	/* Check the scan results using only cached results */
-	if (!(ieee80211_check_scan(vap, IEEE80211_SCAN_NOSSID |
-					IEEE80211_SCAN_KEEPMODE |
-					IEEE80211_SCAN_USECACHE, 0,
-				   vap->iv_des_nssid, vap->iv_des_ssid,
-				   ieee80211_scan_dfs_action))) {
-		/* No channel was found, so call the scan action with no
-		 * result. */
-		ieee80211_scan_dfs_action(vap, NULL);
-	}
+
+	ieee80211_scan_dfs_action(vap);
 }
 
-void
+/* Check if some Non-Occupancy Period timer have expired and update flags
+ * accordingly */
+
+static void
 ieee80211_expire_excl_restrictions(struct ieee80211com *ic)
 {
 	struct ieee80211_channel *c  = NULL;
@@ -868,7 +891,7 @@ ieee80211_expire_excl_restrictions(struct ieee80211com *ic)
 	do_gettimeofday(&tv_now);
 	for (i = 0; i < ic->ic_nchans; i++) {
 		c = &ic->ic_channels[i];
-		if (c->ic_flags & IEEE80211_CHAN_RADAR) {
+		if (IEEE80211_IS_CHAN_RADAR(c)) {
 			if (timeval_compare(&ic->ic_chan_non_occupy[i],
 					    &tv_now) < 0) {
 				if_printf(dev,
@@ -879,6 +902,12 @@ ieee80211_expire_excl_restrictions(struct ieee80211com *ic)
 					  c->ic_ieee, c->ic_freq, tv_now.tv_sec,
 					  tv_now.tv_usec);
 				c->ic_flags &= ~IEEE80211_CHAN_RADAR;
+				ic->ic_chan_non_occupy[i].tv_sec  = 0;
+				ic->ic_chan_non_occupy[i].tv_usec = 0;
+				/* FIXME : should we use ic_curchan or ic_bsschan ? */
+				if (c == ic->ic_curchan) {
+					ic->ic_set_dfs_interference(ic, 0);
+				}
 			} else {
 				if_printf(dev,
 					  "Channel %3d (%4d MHz) is still "
@@ -913,7 +942,7 @@ ieee80211_update_dfs_excl_timer(struct ieee80211com *ic)
 	tv_next.tv_usec = 0;
 	for (i = 0; i < ic->ic_nchans; i++) {
 		chan = &ic->ic_channels[i];
-		if (chan->ic_flags & IEEE80211_CHAN_RADAR) {
+		if (IEEE80211_IS_CHAN_RADAR(chan)) {
 			if ((tv_next.tv_sec == 0) &&
 			    (tv_next.tv_usec == 0)) {
 				tv_next = ic->ic_chan_non_occupy[i];
@@ -935,7 +964,7 @@ ieee80211_update_dfs_excl_timer(struct ieee80211com *ic)
 	}
 }
 
-/* Periodically expire radar avoidance marks. */
+/* Timer callback : periodically expire radar avoidance marks. */
 static void
 ieee80211_expire_dfs_excl_timer(unsigned long data)
 {
@@ -952,17 +981,12 @@ ieee80211_expire_dfs_excl_timer(unsigned long data)
 		/* Go through and clear any interference flag we have, if we
 		 * just got it cleared up for us */
 		TAILQ_FOREACH(vap, &ic->ic_vaps, iv_next) {
-		  /* We need to check for the special value
-		     IEEE80211_CHAN_ANYC before using vap->iv_des_chan
-		     since it will cause a kernel panic */
 			if ((vap->iv_state == IEEE80211_S_RUN) &&
-			    ((vap->iv_opmode == IEEE80211_M_HOSTAP) ||
-			     (vap->iv_opmode == IEEE80211_M_IBSS)) &&
-			    /* Operating on channel other than desired. */
-			    (vap->iv_des_chan != IEEE80211_CHAN_ANYC) &&
-			    (vap->iv_des_chan->ic_freq > 0) &&
-			    (vap->iv_des_chan->ic_freq !=
-			     ic->ic_bsschan->ic_freq)) {
+			    IEEE80211_IS_MODE_DFS_MASTER(vap->iv_opmode) &&
+			    /* Desired channel is really defined */
+			    (vap->iv_des_chan != NULL) &&
+			    (vap->iv_des_chan != IEEE80211_CHAN_ANYC)) {
+
 				struct ieee80211_channel *des_chan =
 					ieee80211_find_channel(ic, 
 							vap->iv_des_chan->
@@ -980,8 +1004,7 @@ ieee80211_expire_dfs_excl_timer(unsigned long data)
 								ic_freq,
 							vap->iv_des_chan->
 								ic_flags);
-				} else if (!(des_chan->ic_flags &
-							IEEE80211_CHAN_RADAR)) {
+				} else if (!IEEE80211_IS_CHAN_RADAR(des_chan)) {
 					IEEE80211_DPRINTF(vap, 
 							IEEE80211_MSG_DOTH,
 							"%s: Desired channel "
@@ -992,11 +1015,25 @@ ieee80211_expire_dfs_excl_timer(unsigned long data)
 								ic_freq,
 							vap->iv_des_chan->
 								ic_flags);
-					ic->ic_chanchange_chan =
-						des_chan->ic_ieee;
-					ic->ic_chanchange_tbtt =
-						IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
-					ic->ic_flags |= IEEE80211_F_CHANSWITCH;
+
+					/* Directly call ic_set_channel() in
+					 * order to start CAC if we are not
+					 * really changing channel (it happens
+					 * at the end of the Non-Occupancy
+					 * Period for instance */
+
+					if (des_chan == ic->ic_bsschan) {
+						ic->ic_curchan = ic->ic_bsschan;
+						ic->ic_set_channel(ic);
+					} else {
+
+					ieee80211_start_new_csa(vap,
+						IEEE80211_CSA_CAN_STOP_TX,
+						des_chan,
+						IEEE80211_DEFAULT_CHANCHANGE_TBTT_COUNT,
+						0);
+					}
+
 				} else if (ieee80211_msg_is_reported(vap, 
 							IEEE80211_MSG_DOTH)) {
 					/* Find the desired channel in 
@@ -1042,7 +1079,7 @@ ieee80211_mark_dfs(struct ieee80211com *ic, struct ieee80211_channel *ichan)
 	struct net_device *dev = ic->ic_dev;
 	struct timeval tv_now;
 	unsigned int excl_period = ic->ic_get_dfs_excl_period(ic);
-	int i;
+	int i, was_on_radar;
 
 	do_gettimeofday(&tv_now);
 	if_printf(dev, "Radar found on channel %3d (%4d MHz) -- "
@@ -1050,6 +1087,11 @@ ieee80211_mark_dfs(struct ieee80211com *ic, struct ieee80211_channel *ichan)
 			ichan->ic_ieee, ichan->ic_freq, 
 			tv_now.tv_sec, tv_now.tv_usec);
 	if (IEEE80211_IS_MODE_DFS_MASTER(ic->ic_opmode)) {
+
+		/* Check if the current channel is already under radar before
+		 * setting radar flag */
+		was_on_radar = IEEE80211_IS_CHAN_RADAR(ic->ic_curchan);
+
 		/* Mark the channel in the ic_chan list */
 		if (ic->ic_flags_ext & IEEE80211_FEXT_MARKDFS) {
 			if_printf(dev, "Marking channel %3d (%4d MHz) in "
@@ -1092,14 +1134,27 @@ ieee80211_mark_dfs(struct ieee80211com *ic, struct ieee80211_channel *ichan)
 				return;
 			}
 			if  (ic->ic_curchan->ic_freq == c->ic_freq) {
-				if_printf(dev, "%s: Invoking "
+
+				/* If current channel is already marked for
+				 * radar, do nothing. It prevents a new CSA
+				 * process to start */
+				if (was_on_radar) {
+					if_printf(dev,
+						"%s: current channel "
+						"already under radar\n",
+						__func__);
+				} else {
+
+					if_printf(dev, "%s: Invoking "
 						"ieee80211_dfs_action "
 						"(%d, 0x%x)\n",
 						__func__, ichan->ic_freq, 
 						ichan->ic_flags);
-				/* The current channel has been marked. We
-				 * need to move away from it. */
-				ieee80211_dfs_action(ic);
+					/* The current channel has been
+					 * marked. We need to move away from
+					 * it. */
+					ieee80211_dfs_action(ic);
+				}
 			} else
 				if_printf(dev,
 						"Unexpected channel frequency! "
@@ -1112,12 +1167,20 @@ ieee80211_mark_dfs(struct ieee80211com *ic, struct ieee80211_channel *ichan)
 						ic->ic_curchan->ic_ieee, 
 						ic->ic_curchan->ic_freq);
 		} else {
+			struct ieee80211vap *vap;
+
 			/* Change to a radar free 11a channel for dfstesttime
-			 * seconds. */
-			ic->ic_chanchange_chan = IEEE80211_RADAR_TEST_MUTE_CHAN;
-			ic->ic_chanchange_tbtt =
-				IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
-			ic->ic_flags |= IEEE80211_F_CHANSWITCH;
+			 * seconds.
+			 * Start a channel switch on all available VAPs. */
+			TAILQ_FOREACH(vap, &ic->ic_vaps, iv_next) {
+				c = ieee80211_doth_findchan(
+						vap,
+						IEEE80211_RADAR_TEST_MUTE_CHAN);
+
+				ieee80211_start_new_csa(vap, IEEE80211_CSA_CAN_STOP_TX,
+					c, IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT, 0);
+			}
+
 			if_printf(dev,
 					"Mute test - markdfs is off, we are "
 					"in hostap mode, found radar on "
@@ -1515,11 +1578,19 @@ enum ieee80211_phymode
 ieee80211_chan2mode(const struct ieee80211_channel *chan)
 {
 	/*
-	 * Callers should handle this case properly, rather than
-	 * just relying that this function returns a sane value.
-	 * XXX: Probably needs to be revised.
+	 * Callers should handle this case properly, rather than just relying
+	 * that this function returns a sane value.  XXX: Probably needs to be
+	 * revised. chan is undefined if channel is 0 for instance and kernel
+	 * panic would happen when called by ieee80211_sta_join1() in IBSS
+	 * mode.
 	 */
-	KASSERT(chan != IEEE80211_CHAN_ANYC, ("channel not setup"));
+
+	if (chan == NULL ||
+	    chan == IEEE80211_CHAN_ANYC) {
+		printk(KERN_ERR "%s: BUG channel not setup: %p\n",
+		       __func__, chan);
+		return IEEE80211_MODE_11B;
+	}
 
 	if (IEEE80211_IS_CHAN_108G(chan))
 		return IEEE80211_MODE_TURBO_G;
@@ -1735,7 +1806,11 @@ ieee80211_set_multicast_list(struct net_device *dev)
 	IEEE80211_UNLOCK_IRQ(ic);
 
 	/* XXX: Merge multicast list into parent device */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
 	parent->set_multicast_list(ic->ic_dev);
+#else
+	parent->netdev_ops->ndo_set_multicast_list(ic->ic_dev);
+#endif
 }
 
 void
