@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_power.c 3204 2008-01-18 17:26:28Z mentor $
+ * $Id: ieee80211_power.c 3713 2008-06-08 01:43:53Z mentor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -52,11 +52,6 @@
 #include <net80211/ieee80211_var.h>
 #include <net80211/ieee80211_proto.h>
 
-#ifndef	NETDEV_TX_OK
-#define	NETDEV_TX_OK	0
-#define	NETDEV_TX_BUSY	1
-#endif
-
 static void ieee80211_set_tim(struct ieee80211_node *ni, int set);
 
 void
@@ -91,7 +86,7 @@ ieee80211_power_latevattach(struct ieee80211vap *vap)
 		MALLOC(vap->iv_tim_bitmap, u_int8_t *, vap->iv_tim_len,
 			M_DEVBUF, M_NOWAIT | M_ZERO);
 		if (vap->iv_tim_bitmap == NULL) {
-			printf("%s: no memory for TIM bitmap!\n", __func__);
+			printk(KERN_ERR "%s: no memory for TIM bitmap!\n", __func__);
 			/* XXX good enough to keep from crashing? */
 			vap->iv_tim_len = 0;
 		}
@@ -120,8 +115,8 @@ ieee80211_node_saveq_drain(struct ieee80211_node *ni)
 	IEEE80211_NODE_SAVEQ_LOCK_IRQ(ni);
 	qlen = IEEE80211_NODE_SAVEQ_QLEN(ni);
 	while ((skb = __skb_dequeue(&ni->ni_savedq)) != NULL) {
-		if (SKB_CB(skb)->ni != NULL)
-			ieee80211_unref_node(&SKB_CB(skb)->ni);
+		if (SKB_NI(skb) != NULL)
+			ieee80211_unref_node(&SKB_NI(skb));
 		ieee80211_dev_kfree_skb(&skb);
 	}
 	IEEE80211_NODE_SAVEQ_UNLOCK_IRQ(ni);
@@ -158,8 +153,8 @@ ieee80211_node_saveq_age(struct ieee80211_node *ni)
 				"discard frame, age %u", M_AGE_GET(skb));
 
 			skb = __skb_dequeue(&ni->ni_savedq);
-			if (SKB_CB(skb)->ni != NULL)
-				ieee80211_unref_node(&SKB_CB(skb)->ni);
+			if (SKB_NI(skb) != NULL)
+				ieee80211_unref_node(&SKB_NI(skb));
 			ieee80211_dev_kfree_skb(&skb);
 			discard++;
 		}
@@ -214,7 +209,7 @@ ieee80211_set_tim(struct ieee80211_node *ni, int set)
 int
 ieee80211_pwrsave(struct sk_buff *skb)
 {
-	struct ieee80211_node *ni = SKB_CB(skb)->ni;
+	struct ieee80211_node *ni = SKB_NI(skb);
 	struct ieee80211vap *vap = ni->ni_vap;
 	struct ieee80211com *ic = ni->ni_ic;
 	struct sk_buff *tail;
@@ -229,11 +224,13 @@ ieee80211_pwrsave(struct sk_buff *skb)
 			ni->ni_stats.ns_psq_drops, IEEE80211_PS_MAX_QUEUE);
 #ifdef IEEE80211_DEBUG
 		if (ieee80211_msg_dumppkts(vap))
-			ieee80211_dump_pkt(ni->ni_ic, skb->data, skb->len, -1, -1);
+			ieee80211_dump_pkt(ni->ni_ic, skb->data,
+					skb->len, -1, -1, 1);
 #endif
-		if (SKB_CB(skb)->ni != NULL)
-			ieee80211_unref_node(&SKB_CB(skb)->ni);
-		return NETDEV_TX_BUSY;
+		if (SKB_NI(skb) != NULL)
+			ieee80211_unref_node(&SKB_NI(skb));
+		ieee80211_dev_kfree_skb(&skb);
+		return NETDEV_TX_OK;
 	}
 
 	/*
@@ -248,7 +245,7 @@ ieee80211_pwrsave(struct sk_buff *skb)
 	tail = skb_peek_tail(&ni->ni_savedq);
 	if (tail != NULL) {
 		age -= M_AGE_GET(tail);
-		__skb_append(tail, skb, &ni->ni_savedq);
+		__skb_queue_after(&ni->ni_savedq, tail, skb);
 	} else
 		__skb_queue_head(&ni->ni_savedq, skb);
 	M_AGE_SET(skb, age);
@@ -355,6 +352,9 @@ ieee80211_sta_pwrsave(struct ieee80211vap *vap, int enable)
 	struct ieee80211_node *ni = vap->iv_bss;
 	int qlen;
 
+	if (vap->iv_opmode != IEEE80211_M_STA)
+		return;
+
 	if (!!enable == !!IEEE80211_VAP_IS_SLEEPING(vap)) /* Bool. normalise */
 		return;
 
